#ifndef TEXTURE_H
#define TEXTURE_H

#include <d3d11.h>

class ContentManager;

/// <summary>
/// Takes care of managing a 2D texture
/// </summary>
class Texture
{
  public:
    Texture(ID3D11ShaderResourceView* texture);
    ~Texture();
    
    /// <summary>
    /// Retrieves the texture
    /// </summary>
    /// <returns>
    /// const pointer to the texture resource view
    /// </returns>
    ID3D11ShaderResourceView* GetTexture() const;
    
  private:
    // disabled
    Texture();
    Texture(const Texture& cpy);
    Texture& operator=(const Texture& cpy);
    
    // not implemented: until the ContentManager tracks the D3D11 resources
    // directly
    friend ContentManager;
    
    /// <summary>
    /// D3D11 version of the texture
    /// </summary>
    ID3D11ShaderResourceView* m_texture;
};

#endif /* TEXTURE_H */
