#ifndef WINDOW_H
#define WINDOW_H

#include <windows.h>

/// <summary>
/// Takes care of creation and management of top-level windows for the
/// application
/// </summary>
class Window
{
  public:
    ~Window();
    
    /// <summary>
    /// Creates a normal overlapped window
    /// </summary>
    /// <param name="title">
    /// title for the window
    /// </param>
    /// <param name="instance">
    /// instance handle to use for creating the window
    /// </param>
    /// <returns>
    /// pointer to the created window
    /// NULL if an error occurred
    /// </returns>
    static Window* CreateNormalWindow(WCHAR* title,HINSTANCE instance);
    
    /// <summary>
    /// Creates a full screen window
    /// </summary>
    /// <param name="title">
    /// title for the window
    /// </param>
    /// <param name="instance">
    /// instance handle to use for creating the window
    /// </param>
    /// <returns>
    /// pointer to the created window
    /// NULL if an error occurred
    /// </returns>
    static Window* CreateFullscreenWindow(WCHAR* title,HINSTANCE instance);
    
    /// <summary>
    /// Retrives the handle for the window
    /// </summary>
    HWND& GetHandle();
    
  private:
    // disabled
    Window(const Window& cpy);
    Window& operator=(const Window& cpy);
    
    // used by static Create functions
    Window();
    
    /// <summary>
    /// handles various messages sent to the window
    /// </summary>
    static LRESULT CALLBACK WndProc(HWND wnd,UINT message,WPARAM wparam,
      LPARAM lparam);
    
    /// <summary>
    /// handle for the instance used to create the window
    /// </summary>
    HINSTANCE m_instance;
    
    /// <summary>
    /// handle for the window
    /// </summary>
    HWND m_wnd;
    
    /// <summary>
    /// class name for the window, needed to unregister it
    /// </summary>
    LPWSTR m_class_name;
};

#endif /* WINDOW_H */
