#include "Effects/VSPSEffect.h"
using namespace std;

VSPSEffect::VSPSEffect(ID3D11VertexShader* vertex,
  ID3D11InputLayout* layout,ID3D11PixelShader* pixel,
  const vector<ID3D11Buffer*>& buffers)
:Effect(),
 m_vertex(vertex),
 m_vertex_layout(layout),
 m_pixel(pixel)
{
  m_num_buffers = buffers.size();
  m_buffers = new ID3D11Buffer*[m_num_buffers];
  vector<ID3D11Buffer*>::const_iterator buff_it = buffers.begin();
  UINT i = 0;
  while (buff_it != buffers.end())
  {
    m_buffers[i] = *buff_it;
    
    ++buff_it;
    ++i;
  }
}

VSPSEffect::~VSPSEffect()
{
  delete m_buffers;
}

void VSPSEffect::MakeActive(ID3D11DeviceContext* context)
{
  // make the vertex shader layout active
  context->IASetInputLayout(m_vertex_layout);
  
  // make the vertex shader active
  context->VSSetShader(m_vertex,NULL,0);
  
  // make the pixel shader active
  context->PSSetShader(m_pixel,NULL,0);
  
  // send the constant buffers to the vertex and pixel shaders
  context->VSSetConstantBuffers(0,m_num_buffers,m_buffers);
  context->PSSetConstantBuffers(0,m_num_buffers,m_buffers);
}

void VSPSEffect::SetConstantBuffer(ID3D11DeviceContext* context,
  UINT id,const void* val)
{
  context->UpdateSubresource(m_buffers[id],0,NULL,val,0,0);
}

void VSPSEffect::UseTexture(ID3D11DeviceContext* context,Texture* tex,
  UINT index)
{
  ID3D11ShaderResourceView* tmp = tex == NULL ? NULL : tex->GetTexture();
  context->PSSetShaderResources(index,1,&tmp);
}

void VSPSEffect::UseTextureSampler(ID3D11DeviceContext* context,
  TextureSampler* sampler,UINT index)
{
  ID3D11SamplerState* tmp = sampler->GetTextureSampler();
  context->PSSetSamplers(index,1,&tmp);
}
