#include "Game.h"
#include "Timer.h"

Game::Game(WCHAR* title,HINSTANCE instance)
:m_mspf(1000 / 60)
{
  // create the game window
//#ifdef _DEBUG
  m_window = Window::CreateNormalWindow(title,instance);
//#else /* _DEBUG */
//  m_window = Window::CreateFullscreenWindow(title,instance);
//#endif /* _DEBUG */
  
  // handle the error case of failing to create the window
  if (!m_window)
  {
    exit(1);
  }
  
  // create the D3D11 core for the window
  m_graphics = D3D11_Core::Create(m_window->GetHandle());
}

Game::~Game()
{
  delete m_graphics;
  delete m_window;
}

void Game::Run()
{
  // perform init
  m_content = new ContentManager(*m_graphics);
  LoadContent();
  
  // get a timer
  Timer* timer = Timer::GetTimer();
  
  // message loop
  MSG msg = {0};
  while(msg.message != WM_QUIT)
  {
    // handle messages
    if (PeekMessage(&msg,NULL,0,0,PM_REMOVE))
    {
      TranslateMessage(&msg);
      DispatchMessage(&msg);
    }
    // check if it's time for another update/render cycle
    else if (timer->CheckDelta(m_mspf))
    {
      Update(m_mspf);
      Render(m_mspf);
    }
  }
  
  // perform cleanup
  UnloadContent();
  delete m_content;
}

void Game::Render(UINT ms)
{
  m_graphics->ClearBackBuffer();
  m_graphics->ClearDepthBuffer();
  
  Draw(ms);
  
  m_graphics->Swap();
}
