#include "Window.h"

Window::Window()
:m_instance(0),
 m_wnd(0),
 m_class_name(NULL)
{
}

Window::~Window()
{
  UnregisterClass(m_class_name,m_instance);
}

Window* Window::CreateNormalWindow(WCHAR* title,HINSTANCE instance)
{
  Window* back       = new Window;
  back->m_instance   = instance;
  back->m_class_name = title;
  
  // register class
  WNDCLASSEX wcex;
  wcex.cbSize        = sizeof(wcex);
  wcex.style         = CS_HREDRAW|CS_VREDRAW;
  wcex.lpfnWndProc   = WndProc;
  wcex.cbClsExtra    = 0;
  wcex.cbWndExtra    = 0;
  wcex.hInstance     = instance;
  wcex.hIcon         = LoadIcon(instance,IDI_APPLICATION);
  wcex.hCursor       = LoadCursor(NULL,IDC_ARROW);
  wcex.hbrBackground = (HBRUSH)COLOR_WINDOW;
  wcex.lpszMenuName  = NULL;
  wcex.lpszClassName = back->m_class_name;
  wcex.hIconSm       = wcex.hIcon;
  if (!RegisterClassEx(&wcex))
  {
    back->m_class_name = NULL;
    delete back;
    return NULL;
  }

  // create window
  RECT rc = { 0, 0, 640, 480 };
  AdjustWindowRect(&rc,WS_OVERLAPPEDWINDOW,FALSE);
  back->m_wnd = CreateWindow(wcex.lpszClassName,title,WS_OVERLAPPEDWINDOW,
    CW_USEDEFAULT,CW_USEDEFAULT,rc.right - rc.left,rc.bottom - rc.top,NULL,
    NULL,instance,NULL);
  if (!back->m_wnd)
  {
    delete back;
    return NULL;
  }

  // display the window
  ShowWindow(back->m_wnd,1);

  return back;
}

Window* Window::CreateFullscreenWindow(WCHAR* title,HINSTANCE instance)
{
  Window* back       = new Window;
  back->m_instance   = instance;
  back->m_class_name = title;
  
  // register class
  WNDCLASSEX wcex;
  wcex.cbSize        = sizeof(wcex);
  wcex.style         = CS_HREDRAW|CS_VREDRAW;
  wcex.lpfnWndProc   = WndProc;
  wcex.cbClsExtra    = 0;
  wcex.cbWndExtra    = 0;
  wcex.hInstance     = instance;
  wcex.hIcon         = LoadIcon(instance,IDI_APPLICATION);
  wcex.hCursor       = LoadCursor(NULL,IDC_ARROW);
  wcex.hbrBackground = (HBRUSH)COLOR_WINDOW;
  wcex.lpszMenuName  = NULL;
  wcex.lpszClassName = back->m_class_name;
  wcex.hIconSm       = wcex.hIcon;
  if (!RegisterClassEx(&wcex))
  {
    back->m_class_name = NULL;
    delete back;
    return NULL;
  }
  
  HMONITOR monitor = MonitorFromWindow(NULL,MONITOR_DEFAULTTONEAREST);
  MONITORINFO info;
  info.cbSize = sizeof(info);
  if (!GetMonitorInfo(monitor,&info))
  {
    delete back;
    return NULL;
  }

  // create window
  back->m_wnd = CreateWindow(wcex.lpszClassName,title,
    WS_POPUP|WS_VISIBLE,info.rcMonitor.left,info.rcMonitor.top,
    info.rcMonitor.right - info.rcMonitor.left,
    info.rcMonitor.bottom - info.rcMonitor.top,NULL,
    NULL,instance,NULL);
  if (!back->m_wnd)
  {
    delete back;
    return NULL;
  }

  // display the window
  ShowWindow(back->m_wnd,1);

  return back;
}

HWND& Window::GetHandle()
{
  return m_wnd;
}

LRESULT CALLBACK Window::WndProc(HWND wnd,UINT message,WPARAM wparam,
  LPARAM lparam)
{
  PAINTSTRUCT ps;

  switch(message)
  {
    case WM_PAINT:
      BeginPaint(wnd,&ps);
      EndPaint(wnd,&ps);
      break;

    case WM_DESTROY:
      PostQuitMessage(0);
      break;

    default:
      return DefWindowProc(wnd,message,wparam,lparam);
  }

  return 0;
}
