#include <vector>
#include "GameMain.h"
#include <xnamath.h>
#include "VertexDeclarations/Vertex_PositionTexture.h"
using namespace std;

GameMain::GameMain(WCHAR* title,HINSTANCE instance)
:Game(title,instance),
 m_effect(NULL),
 m_verts(NULL),
 m_indices(NULL),
 m_instance(NULL),
 m_texture(NULL),
 m_sampler(NULL),
 m_vert_array(NULL)
{
}

GameMain::~GameMain()
{
}

void GameMain::LoadContent()
{
  D3D11_INPUT_ELEMENT_DESC layout[] =
  {
    { "POSITION", 0, DXGI_FORMAT_R32G32B32_FLOAT, 0, 0,
      D3D11_INPUT_PER_VERTEX_DATA, 0 },
    { "TEXCOORD", 0, DXGI_FORMAT_R32G32_FLOAT, 0, 12,
      D3D11_INPUT_PER_VERTEX_DATA, 0 },
    { "TEXCOORD", 1, DXGI_FORMAT_R32G32B32_FLOAT, 1, 0,
      D3D11_INPUT_PER_INSTANCE_DATA, 1 },
  };
  
  vector<UINT> buffer_widths;
  buffer_widths.push_back(sizeof(XMMATRIX));
  
  m_effect = m_content->LoadEffect(L"Tutorial07.fx",SM_4_0,layout,
    ARRAYSIZE(layout),buffer_widths);
  if (m_effect == NULL)
  {
    exit(1);
  }
  
  // set the wvp matrix for the effect (note: done here since it doesn't change
  // in this particular test)
  D3D11_VIEWPORT viewport = m_graphics->GetViewport(0);
  XMMATRIX wvp = XMMatrixIdentity();
  XMVECTOR pos = XMVectorSet(0.0f, 0.0f,-10.0f, 1.0f);
  XMVECTOR at  = XMVectorSet(0.0f, 0.0f,  0.0f, 1.0f);
  XMVECTOR up  = XMVectorSet(0.0f, 1.0f,  0.0f, 0.0f);
  wvp *= XMMatrixLookAtLH(pos,at,up);
  wvp *= XMMatrixPerspectiveFovLH(XM_PIDIV4,
    viewport.Width / viewport.Height,0.01f,100.0f);
  wvp = XMMatrixTranspose(wvp);
  m_effect->SetConstantBuffer(m_graphics->GetImmediateContext(),0,&wvp);
  
  // create the vertex buffer
  /*
  XMFLOAT3 vertices[] =
  {
    XMFLOAT3(-1.0f, 1.0f,-1.0f),
    XMFLOAT3( 1.0f, 1.0f,-1.0f),
    XMFLOAT3( 1.0f, 1.0f, 1.0f),
    XMFLOAT3(-1.0f, 1.0f, 1.0f),

    XMFLOAT3(-1.0f,-1.0f,-1.0f),
    XMFLOAT3( 1.0f,-1.0f,-1.0f),
    XMFLOAT3( 1.0f,-1.0f, 1.0f),
    XMFLOAT3(-1.0f,-1.0f, 1.0f),

    XMFLOAT3(-1.0f,-1.0f, 1.0f),
    XMFLOAT3(-1.0f,-1.0f,-1.0f),
    XMFLOAT3(-1.0f, 1.0f,-1.0f),
    XMFLOAT3(-1.0f, 1.0f, 1.0f),

    XMFLOAT3( 1.0f,-1.0f, 1.0f),
    XMFLOAT3( 1.0f,-1.0f,-1.0f),
    XMFLOAT3( 1.0f, 1.0f,-1.0f),
    XMFLOAT3( 1.0f, 1.0f, 1.0f),

    XMFLOAT3(-1.0f,-1.0f,-1.0f),
    XMFLOAT3( 1.0f,-1.0f,-1.0f),
    XMFLOAT3( 1.0f, 1.0f,-1.0f),
    XMFLOAT3(-1.0f, 1.0f,-1.0f),

    XMFLOAT3(-1.0f,-1.0f, 1.0f),
    XMFLOAT3( 1.0f,-1.0f, 1.0f),
    XMFLOAT3( 1.0f, 1.0f, 1.0f),
    XMFLOAT3(-1.0f, 1.0f, 1.0f),
  };
  */
  Vertex_PositionTexture vertices[] =
  {
    { XMFLOAT3(-1.0f, 1.0f,-1.0f),XMFLOAT2(0.0f,0.0f) },
    { XMFLOAT3( 1.0f, 1.0f,-1.0f),XMFLOAT2(1.0f,0.0f) },
    { XMFLOAT3( 1.0f, 1.0f, 1.0f),XMFLOAT2(1.0f,1.0f) },
    { XMFLOAT3(-1.0f, 1.0f, 1.0f),XMFLOAT2(0.0f,1.0f) },

    { XMFLOAT3(-1.0f,-1.0f,-1.0f),XMFLOAT2(0.0f,0.0f) },
    { XMFLOAT3( 1.0f,-1.0f,-1.0f),XMFLOAT2(1.0f,0.0f) },
    { XMFLOAT3( 1.0f,-1.0f, 1.0f),XMFLOAT2(1.0f,1.0f) },
    { XMFLOAT3(-1.0f,-1.0f, 1.0f),XMFLOAT2(0.0f,1.0f) },

    { XMFLOAT3(-1.0f,-1.0f, 1.0f),XMFLOAT2(0.0f,0.0f) },
    { XMFLOAT3(-1.0f,-1.0f,-1.0f),XMFLOAT2(1.0f,0.0f) },
    { XMFLOAT3(-1.0f, 1.0f,-1.0f),XMFLOAT2(1.0f,1.0f) },
    { XMFLOAT3(-1.0f, 1.0f, 1.0f),XMFLOAT2(0.0f,1.0f) },

    { XMFLOAT3( 1.0f,-1.0f, 1.0f),XMFLOAT2(0.0f,0.0f) },
    { XMFLOAT3( 1.0f,-1.0f,-1.0f),XMFLOAT2(1.0f,0.0f) },
    { XMFLOAT3( 1.0f, 1.0f,-1.0f),XMFLOAT2(1.0f,1.0f) },
    { XMFLOAT3( 1.0f, 1.0f, 1.0f),XMFLOAT2(0.0f,1.0f) },

    { XMFLOAT3(-1.0f,-1.0f,-1.0f),XMFLOAT2(0.0f,0.0f) },
    { XMFLOAT3( 1.0f,-1.0f,-1.0f),XMFLOAT2(1.0f,0.0f) },
    { XMFLOAT3( 1.0f, 1.0f,-1.0f),XMFLOAT2(1.0f,1.0f) },
    { XMFLOAT3(-1.0f, 1.0f,-1.0f),XMFLOAT2(0.0f,1.0f) },

    { XMFLOAT3(-1.0f,-1.0f, 1.0f),XMFLOAT2(0.0f,0.0f) },
    { XMFLOAT3( 1.0f,-1.0f, 1.0f),XMFLOAT2(1.0f,0.0f) },
    { XMFLOAT3( 1.0f, 1.0f, 1.0f),XMFLOAT2(1.0f,1.0f) },
    { XMFLOAT3(-1.0f, 1.0f, 1.0f),XMFLOAT2(0.0f,1.0f) },
  };
  m_verts = VertexBuffer_PositionTexture::Create(*m_graphics,
    ARRAYSIZE(vertices),vertices);
  if (m_verts == NULL)
  {
    exit(1);
  }
  
  // create the index buffer
  WORD indices[] =
  {
    3,1,0,
    2,1,3,

    6,4,5,
    7,4,6,

    11,9,8,
    10,9,11,

    14,12,13,
    15,12,14,

    19,17,16,
    18,17,19,

    22,20,21,
    23,20,22
  };
  m_indices = IndexBuffer16::Create(*m_graphics,ARRAYSIZE(indices),indices);
  if (m_indices == NULL)
  {
    exit(1);
  }
  
  // create the instance buffer
  XMFLOAT3 instance_pos[] =
  {
    XMFLOAT3(-2,0,0),
    XMFLOAT3( 2,0,0),
  };
  m_instance = InstanceBuffer::Create(*m_graphics,
    sizeof(instance_pos) / sizeof(XMFLOAT3),sizeof(XMFLOAT3),instance_pos);
  if (m_instance == NULL)
  {
    exit(1);
  }
  
  // combine the vertex and instance buffer
  m_vert_array = new VertexBufferArray(2);
  m_vert_array->Set(0,*m_verts);
  m_vert_array->Set(1,*m_instance);

  // load the test texture
  m_texture = m_content->LoadTexture(L"seafloor.dds");
  if (m_texture == NULL)
  {
    exit(1);
  }
  
  // create the texture sampler
  D3D11_SAMPLER_DESC sample_desc;
  ZeroMemory(&sample_desc,sizeof(sample_desc));
  sample_desc.Filter         = D3D11_FILTER_MIN_MAG_MIP_LINEAR;
  sample_desc.AddressU       = D3D11_TEXTURE_ADDRESS_WRAP;
  sample_desc.AddressV       = D3D11_TEXTURE_ADDRESS_WRAP;
  sample_desc.AddressW       = D3D11_TEXTURE_ADDRESS_WRAP;
  sample_desc.ComparisonFunc = D3D11_COMPARISON_NEVER;
  sample_desc.MinLOD         = 0;
  sample_desc.MaxLOD         = D3D11_FLOAT32_MAX;
  m_sampler = TextureSampler::Create(sample_desc,*m_graphics);
  if (m_sampler == NULL)
  {
    exit(1);
  }
}

void GameMain::UnloadContent()
{
  delete m_instance;
  delete m_sampler;
  delete m_verts;
  delete m_indices;
  m_content->UnloadTexture(m_texture);
  m_content->UnloadEffect(m_effect);
}

void GameMain::Update(UINT ms)
{
}

void GameMain::Draw(UINT ms)
{
  // only using 1 d3d11 thread, so just get the immediate context
  ID3D11DeviceContext* context = m_graphics->GetImmediateContext();
  
  // apply the effect
  m_effect->MakeActive(context);
  
  // use the texture and sampler on the effect
  m_effect->UseTexture(context,m_texture,0);
  m_effect->UseTextureSampler(context,m_sampler,0);
  
  // draw the test model
  m_vert_array->MakeActive(context);
  m_indices->MakeActive(context);
  m_graphics->DrawInstancedPrimitives(context,
    D3D11_PRIMITIVE_TOPOLOGY_TRIANGLELIST,0,36,0,2);
}
