#ifndef GAMEMAIN_H
#define GAMEMAIN_H

#include "Game.h"
#include "Effects/VSPSEffect.h"
#include "Buffers/VertexBuffer_PositionTexture.h"
#include "Buffers/IndexBuffer16.h"
#include "Buffers/InstanceBuffer.h"
#include "Buffers/VertexBufferArray.h"
#include "Texture.h"
#include "TextureSampler.h"

class GameMain : public Game
{
  public:
    GameMain(WCHAR* title,HINSTANCE instance);
    ~GameMain();
    
  protected:
    /// <summary>
    /// Loads the content for the game
    /// </summary>
    /// <param name="content">
    /// content manager to use for loading resources
    /// </param>
    void LoadContent();
    
    /// <summary>
    /// Unloads the content for the game before the dealloction of the instance
    /// </summary>
    /// <param name="content">
    /// content manager to use for unloading resources
    /// </param>
    void UnloadContent();
    
    /// <summary>
    /// Updates the game state
    /// </summary>
    /// <param name="ms">
    /// number of milliseconds since the update
    /// </param>
    void Update(UINT ms);
    
    /// <summary>
    /// Draws the current frame
    /// </summary>
    /// <param name="ms">
    /// number of milliseconds since the last frame
    /// </param>
    void Draw(UINT ms);
    
  private:
    // disabled
    GameMain(const GameMain& cpy);
    GameMain& operator=(const GameMain& cpy);
    
    /// <summary>
    /// test effect
    /// </summary>
    VSPSEffect* m_effect;

    /// <summary>
    /// vertex buffer for the test cube
    /// </summary>
    VertexBuffer_PositionTexture* m_verts;

    /// <summary>
    /// index buffer for the test cube
    /// </summary>
    IndexBuffer16* m_indices;

    /// <summary>
    /// instance buffer for the test cube
    /// </summary>
    InstanceBuffer* m_instance;
    
    /// <summary>
    /// texture to apply to the cube
    /// </summary>
    Texture* m_texture;
    
    /// <summary>
    /// texture sampler to use
    /// </summary>
    TextureSampler* m_sampler;
    
    /// <summary>
    /// vertex buffer array for supplying the vertex and instance buffers to
    /// the rendering process
    /// </summary>
    VertexBufferArray* m_vert_array;
};

#endif /* GAMEMAIN_H */
